// ------------------------------------------------------------------------------------
// <copyright file="UsuariosBL.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.Negocio.Clases.BL
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Biblioteca.Datos.Clases.DAL.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.IC.DTO.Repositorio;
	using Biblioteca.IC.Recursos;
	using TransaccionalNet.EF.Clases;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using Utilitarios.TransversalesNet.Clases.Comunes.Enumeraciones;

	/// <summary>
	/// Clase con las acciones de negocio de la entidad Usuarios
	/// </summary>
	public class UsuariosBL : AccesoComunBL, IUsuariosNegocioAccion
	{

		#region ATRIBUTOS
		/// <summary>
		/// Objeto para repositorio Usuarios
		/// <summary>
		/// Objeto para acciones Usuarios
		/// </summary>
		private Lazy<IUsuariosRepositorioAccion> usuariosRepositorioAccion;

		/// <summary>
		/// Objeto para entidad respuesta
		/// </summary>
		Respuesta<IUsuariosDTO> respuesta;

		#endregion ATRIBUTOS
		#region CONSTRUCTORES
		/// <summary>
		/// Inicializa una nueva instancia de la clase UsuariosBL
		/// </summary>
		/// <param name="argUsuariosRepositorioAccion">Acciones entidad Usuarios</param>
		public UsuariosBL(Lazy<IUsuariosRepositorioAccion> argUsuariosRepositorioAccion = null)
		{
			this.respuesta = new Respuesta<IUsuariosDTO>();
			this.usuariosRepositorioAccion = argUsuariosRepositorioAccion ?? new Lazy<IUsuariosRepositorioAccion>(() => new UsuariosDAL());
		}

		#endregion CONSTRUCTORES
		#region METODOS PUBLICOS
		/// <summary>
		/// Metodo consultar lista usuarios
		/// </summary>
		/// <returns>Respuesta tipo Usuarios </returns>
		public async Task<Respuesta<IUsuariosDTO>> ConsultarListaUsuariosAsync()
		{

			return await this.EjecutarTransaccionBDAsync<Respuesta<IUsuariosDTO>, UsuariosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.usuariosRepositorioAccion.Value.ConsultarListaUsuariosAsync();
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});

		}
		/// <summary>
		/// Metodo consultar por llave usuarios
		/// </summary>
		/// <param name="usuarios">Entidad a consultar</param>
		/// <returns>Respuesta tipo Usuarios </returns>
		public async Task<Respuesta<IUsuariosDTO>> ConsultarUsuariosLlaveAsync(IUsuariosDTO usuarios)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IUsuariosDTO>, UsuariosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.usuariosRepositorioAccion.Value.ConsultarUsuariosLlaveAsync(usuarios);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo editar usuarios
		/// </summary>
		/// <param name="usuarios">Entidad a editar</param>
		/// <returns>Respuesta tipo Usuarios </returns>
		public async Task<Respuesta<IUsuariosDTO>> EditarUsuariosAsync(IUsuariosDTO usuarios)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IUsuariosDTO>, UsuariosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.usuariosRepositorioAccion.Value.EditarUsuariosAsync(usuarios);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}


		/// <summary>
		/// Metodo eliminar usuarios
		/// </summary>
		/// <param name="usuarios">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Usuarios </returns>
		public async Task<Respuesta<IUsuariosDTO>> EliminarUsuariosAsync(IUsuariosDTO usuarios)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IUsuariosDTO>, UsuariosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.usuariosRepositorioAccion.Value.EliminarUsuariosAsync(usuarios);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeEntidadEliminadaConExito);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo guardar usuarios
		/// </summary>
		/// <param name="usuarios">Entidad a guardar</param>
		/// <returns>Respuesta tipo Usuarios </returns>
		public async Task<Respuesta<IUsuariosDTO>> GuardarUsuariosAsync(IUsuariosDTO usuarios)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IUsuariosDTO>, UsuariosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.usuariosRepositorioAccion.Value.GuardarUsuariosAsync(usuarios);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo Guardar lista usuarios
		/// </summary>
		/// <param name="listaUsuarios">Lista de entidades a guardar</param>
		/// <returns>Respuesta tipo Usuarios </returns>
		public async Task<Respuesta<IUsuariosDTO>> GuardarListaUsuariosAsync(List<IUsuariosDTO> listaUsuarios)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IUsuariosDTO>, UsuariosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.usuariosRepositorioAccion.Value.GuardarListaUsuariosAsync(listaUsuarios);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}
		#endregion METODOS PUBLICOS
		#region METODOS PRIVADOS
		#endregion METODOS PRIVADOS
	}
}

