// ------------------------------------------------------------------------------------
// <copyright file="UsuariosController.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.API.Controllers
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Utilitarios.APINet.Clases.Ejecucion;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using UtilitariosNet.Clases.Mapeador.Extensiones;
	using Biblioteca.API.Models.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.Negocio.Clases.BL;
	using Microsoft.AspNetCore.Mvc;
	
	/// <summary>
	/// Clase con las capaciades Rest de la entidad Usuarios
	/// </summary>
	[Route("api/Usuarios")]
	public class UsuariosController : AccesoComunAPI
	{


		/// <summary>
		/// Objeto para negocio Usuarios
		/// </summary>
		private Lazy<IUsuariosNegocioAccion> negocioUsuarios;

		
		/// <summary>
		/// Inicializa una nueva instancia de la clase UsuariosController
		/// </summary>
		public UsuariosController()
		{
			this.negocioUsuarios = new Lazy<IUsuariosNegocioAccion>(
										() => new UsuariosBL());
		}
		
		/// <summary>
		/// Metodo consultar lista usuarios
		/// </summary>
		/// <returns>Respuesta tipo Usuarios </returns>
		[HttpGet]
		[Route("ConsultarListaUsuarios")]
		public async Task<Respuesta<Usuarios>> ConsultarListaUsuarios()
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Usuarios>>, UsuariosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Usuarios>>(await negocioUsuarios.Value.ConsultarListaUsuariosAsync());
			});
		}
		
		/// <summary>
		/// Metodo consultar por llave usuarios
		/// </summary>
		/// <param name="usuarios">Entidad a consultar</param>
		/// <returns>Respuesta tipo Usuarios </returns>
		[HttpPost]
		[Route("ConsultarUsuariosLlave")]
		public async Task<Respuesta<Usuarios>> ConsultarUsuariosLlave(Usuarios usuarios)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Usuarios>>, UsuariosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Usuarios>>(await negocioUsuarios.Value.ConsultarUsuariosLlaveAsync(usuarios));
			});
		}
		
		/// <summary>
		/// Metodo guardar usuarios
		/// </summary>
		/// <param name="usuarios">Entidad a guardar</param>
		/// <returns>Respuesta tipo Usuarios </returns>
		[HttpPost]
		[Route("GuardarUsuarios")]
		public async Task<Respuesta<Usuarios>> GuardarUsuarios(Usuarios usuarios)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Usuarios>>, UsuariosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Usuarios>>(await negocioUsuarios.Value.GuardarUsuariosAsync(usuarios));
			});
		}
		
		/// <summary>
		/// Metodo editar usuarios
		/// </summary>
		/// <param name="usuarios">Entidad a editar</param>
		/// <returns>Respuesta tipo Usuarios </returns>
		[HttpPut]
		[Route("EditarUsuarios")]
		public async Task<Respuesta<Usuarios>> EditarUsuarios(Usuarios usuarios)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Usuarios>>, UsuariosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Usuarios>>(await negocioUsuarios.Value.EditarUsuariosAsync(usuarios));
			});
		}
		
		/// <summary>
		/// Metodo eliminar usuarios
		/// </summary>
		/// <param name="usuarios">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Usuarios </returns>
		[HttpDelete]
		[Route("EliminarUsuarios")]
		public async Task<Respuesta<Usuarios>> EliminarUsuarios(Usuarios usuarios)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Usuarios>>, UsuariosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Usuarios>>(await negocioUsuarios.Value.EliminarUsuariosAsync(usuarios));
			});
		}
	}
}

